@@ -28,19 +28,48 @@
 
 #include <qmldesignerplugin.h>
 
+#include <coreplugin/icore.h>
+
 #include <utils/stylehelper.h>
 
 #include <QApplication>
-#include <QRegExp>
+#include <QRegularExpression>
 #include <QScreen>
 #include <QPointer>
+#include <QQmlEngine>
+#include <QQmlComponent>
+#include <QQmlProperty>
 #include <qqml.h>
 
+static Q_LOGGING_CATEGORY(themeLog, "qtc.qmldesigner.theme", QtWarningMsg)
+
 namespace QmlDesigner {
 
 Theme::Theme(Utils::Theme *originTheme, QObject *parent)
     : Utils::Theme(originTheme, parent)
+    , m_constants(nullptr)
 {
+    QString constantsPath
+        = Core::ICore::resourcePath(
+                  "qmldesigner/propertyEditorQmlSources/imports/StudioTheme/InternalConstants.qml")
+              .toString();
+
+    QQmlEngine* engine = new QQmlEngine(this);
+    QQmlComponent component(engine, QUrl::fromLocalFile(constantsPath));
+
+    if (component.status() == QQmlComponent::Ready) {
+        m_constants = component.create();
+    }
+    else if (component.status() == QQmlComponent::Error ) {
+        qCWarning(themeLog) << "Couldn't load" << constantsPath
+                            << "due to the following error(s):";
+        for (const QQmlError &error : component.errors())
+            qCWarning(themeLog) << error.toString();
+    }
+    else {
+        qCWarning(themeLog) << "Couldn't load" << constantsPath
+                            << "the status of the QQmlComponent is" << component.status();
+    }
 }
 
 QColor Theme::evaluateColorAtThemeInstance(const QString &themeColorName)
@@ -49,7 +78,7 @@ QColor Theme::evaluateColorAtThemeInstance(const QString &themeColorName)
     const QMetaEnum e = m.enumerator(m.indexOfEnumerator("Color"));
     for (int i = 0, total = e.keyCount(); i < total; ++i) {
         if (QString::fromLatin1(e.key(i)) == themeColorName)
-            return color(static_cast<Utils::Theme::Color>(i)).name();
+            return color(static_cast<Utils::Theme::Color>(i));
     }
 
     qWarning() << Q_FUNC_INFO << "error while evaluating" << themeColorName;
@@ -65,23 +94,32 @@ Theme *Theme::instance()
 
 QString Theme::replaceCssColors(const QString &input)
 {
-    QRegExp rx("creatorTheme\\.(\\w+)");
+    const QRegularExpression rx("creatorTheme\\.(\\w+)");
 
     int pos = 0;
     QString output = input;
 
-    while ((pos = rx.indexIn(input, pos)) != -1) {
-        const QString themeColorName = rx.cap(1);
+    QRegularExpressionMatchIterator it = rx.globalMatch(input);
+    while (it.hasNext()) {
+        const QRegularExpressionMatch match = it.next();
+        const QString themeColorName = match.captured(1);
+        const QRegularExpression replaceExp("creatorTheme\\." + themeColorName + "(\\s|;|\\n)");
 
         if (themeColorName == "smallFontPixelSize") {
-            output.replace("creatorTheme." + themeColorName, QString::number(instance()->smallFontPixelSize()) + "px");
+            output.replace(replaceExp,
+                           QString::number(instance()->smallFontPixelSize()) + "px" + "\\1");
         } else if (themeColorName == "captionFontPixelSize") {
-            output.replace("creatorTheme." + themeColorName, QString::number(instance()->captionFontPixelSize()) + "px");
+            output.replace(replaceExp,
+                           QString::number(instance()->captionFontPixelSize()) + "px" + "\\1");
         } else {
             const QColor color = instance()->evaluateColorAtThemeInstance(themeColorName);
-            output.replace("creatorTheme." + rx.cap(1), color.name());
+            // Create rgba(r, g, b, a)
+            const QString rgbaStr = QString("rgba(%1, %2, %3, %4)")
+                                    .arg(color.red()).arg(color.green()).arg(color.blue())
+                                    .arg(color.alpha());
+            output.replace(replaceExp, rgbaStr + "\\1");
         }
-        pos += rx.matchedLength();
+        pos += match.capturedLength();
     }
 
     return output;
@@ -89,10 +127,10 @@ QString Theme::replaceCssColors(const QString &input)
 
 void Theme::setupTheme(QQmlEngine *engine)
 {
-    static const int typeIndex = qmlRegisterSingletonType<Utils::Theme>("QtQuickDesignerTheme", 1, 0,
-        "Theme", [](QQmlEngine *, QJSEngine *) {
-            return qobject_cast<QObject*>(new Theme(Utils::creatorTheme(), nullptr));
-    });
+    static const int typeIndex = qmlRegisterSingletonType<Theme>(
+        "QtQuickDesignerTheme", 1, 0, "Theme", [](QQmlEngine *, QJSEngine *) {
+            return new Theme(Utils::creatorTheme(), nullptr);
+        });
     Q_UNUSED(typeIndex)
 
     engine->addImageProvider(QLatin1String("icons"), new QmlDesignerIconProvider());
@@ -129,6 +167,25 @@ QPixmap Theme::getPixmap(const QString &id)
     return QmlDesignerIconProvider::getPixmap(id);
 }
 
+QString Theme::getIconUnicode(Theme::Icon i)
+{
+    if (!instance()->m_constants)
+        return QString();
+
+    const QMetaObject *m = instance()->metaObject();
+    const char *enumName = "Icon";
+    int enumIndex = m->indexOfEnumerator(enumName);
+
+    if (enumIndex == -1) {
+        qCWarning(themeLog) << "Couldn't find enum" << enumName;
+        return QString();
+    }
+
+    QMetaEnum e = m->enumerator(enumIndex);
+
+    return instance()->m_constants->property(e.valueToKey(i)).toString();
+}
+
 QColor Theme::qmlDesignerBackgroundColorDarker() const
 {
     return getColor(QmlDesigner_BackgroundColorDarker);
